@@ -27,9 +27,13 @@
      Fixes:
 
      Add -r flag: Keith Owens <kaos@ocs.com.au> October 1999.
+ 
+     Rationalize common code for 32/64 bit architectures.
+     Fix error message for modules.dep.
+       Keith Owens <kaos@ocs.com.au> December 1999
    */
 
-#ident "$Id: depmod.c 1.5 Tue, 07 Dec 1999 21:42:01 +1100 keith $"
+#ident "$Id: depmod.c 1.7 Mon, 20 Dec 1999 12:34:51 +1100 keith $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,14 +58,6 @@
 #include "config.h"
 #include "modstat.h"
 
-#if defined(COMMON_sparc)
-#if defined(ARCH_sparc)
-#define main depmod32_main
-#elif defined(ARCH_sparc64)
-#define main depmod64_main
-#endif
-#endif
-
 #define ALLOC_MODULE	5000	/* Number of modules we can handle... */
 #define ALLOC_SYM	10000	/* Number of symbols allocated per chunk */
 
@@ -106,6 +102,11 @@ static int quiet;	/* Don't print errors */
 static int showerror;	/* Shows undefined symbols */
 static int verbose;	/* Print all modules visiteds */
 
+extern int depmod_main(int argc, char **argv);
+extern int depmod_main_32(int argc, char **argv);
+extern int depmod_main_64(int argc, char **argv);
+
+
 /*
  *	Create a symbol definition.
  *	Add defined symbol to the head of the list of defined symbols.
@@ -486,7 +487,7 @@ static void prtdepend(const char *pdepfile, char *base_dir)
 
 	if (pdepfile != NULL) {
 		if ((out = fopen(pdepfile, "w")) == NULL) {
-			error("Can't open %s", pdepfile);
+			error("Can't open %s for writing", pdepfile);
 			exit(-1);
 		}
 	}
@@ -546,7 +547,11 @@ static void prtdepend(const char *pdepfile, char *base_dir)
 	}
 }
 
-static void usage()
+/* For common 3264, only compile the usage message once, in the 64 bit version */
+#if defined(COMMON_3264) && defined(ONLY_32)
+extern void depmod_usage(void);         /* Use the copy in the 64 bit version */
+#else   /* Common 64 bit version or any non common code - compile usage routine */
+void depmod_usage(void)
 {
 	fprintf(stderr,
 	"depmod " MODUTILS_VERSION "\n"
@@ -576,8 +581,29 @@ static void usage()
 	"Option '-F kernelsyms': use the file instead of the current kernel symbols.\n"
 	);
 }
+#endif  /* defined(COMMON_3264) && defined(ONLY_32) */
 
-int main(int argc, char *argv[])
+#if defined(COMMON_3264) && defined(ONLY_32)
+#define DEPMOD_MAIN depmod_main_32	/* 32 bit version */
+#elif defined(COMMON_3264) && defined(ONLY_64)
+#define DEPMOD_MAIN depmod_main_64	/* 64 bit version */
+#else
+#define DEPMOD_MAIN depmod_main		/* Not common 3264 code */
+#endif
+
+/* For common 3264 code, add an overall depmod_main, in the 64 bit version. */
+#if defined(COMMON_3264) && defined(ONLY_64)
+int depmod_main(int argc, char **argv)
+{
+        struct utsname u;
+        if (!uname (&u) && !strcmp (u.machine, "64"))
+                return depmod_main_64(argc, argv);
+        else
+                return depmod_main_32(argc, argv);
+}
+#endif  /* defined(COMMON_3264) && defined(ONLY_64) */
+
+int DEPMOD_MAIN(int argc, char **argv)
 {
 	int ret = -1;
 	int stdmode = 0;
@@ -623,7 +649,7 @@ int main(int argc, char *argv[])
 
 		case '?':
 		case 'h':
-			usage();
+			depmod_usage();
 			return 0;
 			break;
 
@@ -709,20 +735,12 @@ int main(int argc, char *argv[])
 	return ret;
 }
 
-#if defined(COMMON_sparc) && defined(ARCH_sparc)
-
-#undef main
-
-extern int depmod64_main(int argc, char **argv);
-
-int
-main(int argc, char **argv)
+/* For common 3264 code, only compile main in the 64 bit version. */
+#if defined(COMMON_3264) && defined(ONLY_32)
+/* Use the main in the 64 bit version */
+#else
+int main(int argc, char **argv)
 {
-	struct utsname u;
-  
-	if (!uname (&u) && !strcmp (u.machine, "sparc64"))
-		return depmod64_main(argc, argv);
-	else
-		return depmod32_main(argc, argv);
+	return depmod_main(argc, argv);
 }
-#endif
+#endif	/* defined(COMMON_3264) && defined(ONLY_32) */
