@@ -24,7 +24,7 @@
 #ifndef MODUTILS_OBJ_H
 #define MODUTILS_OBJ_H 1
 
-#ident "$Id: obj.h 1.1 Wed, 25 Aug 1999 16:26:49 +1000 keith $"
+#ident "$Id: obj.h 1.2 Mon, 22 Nov 1999 01:38:27 +1100 keith $"
 
 /* The relocatable object is manipulated using elfin types.  */
 
@@ -42,7 +42,17 @@
 # endif
 #endif
 
-/* For some reason this is missing from libc5.  */
+#if defined(__sparc__) && !defined(__sparc_v9__)
+# if ELFCLASSM == ELFCLASS32
+#  define ObjW(x)  obj32_ ## x
+# else
+#  define ObjW(x)  obj64_ ## x
+# endif
+#else
+#define ObjW(x)    obj_ ## x
+#endif
+
+/* For some reason this is missing from lib5.  */
 #ifndef ELF32_ST_INFO
 # define ELF32_ST_INFO(bind, type)       (((bind) << 4) + ((type) & 0xf))
 #endif
@@ -51,8 +61,8 @@
 # define ELF64_ST_INFO(bind, type)       (((bind) << 4) + ((type) & 0xf))
 #endif
 
-struct obj_string_patch;
-struct obj_symbol_patch;
+struct obj_string_patch_struct;
+struct obj_symbol_patch_struct;
 
 struct obj_section
 {
@@ -88,8 +98,8 @@ struct obj_file
   struct obj_section **sections;
   struct obj_section *load_order;
   struct obj_section **load_order_search_start;
-  struct obj_string_patch *string_patches;
-  struct obj_symbol_patch *symbol_patches;
+  struct obj_string_patch_struct *string_patches;
+  struct obj_symbol_patch_struct *symbol_patches;
   int (*symbol_cmp)(const char *, const char *);
   unsigned long (*symbol_hash)(const char *);
   unsigned long local_symtab_size;
@@ -105,17 +115,17 @@ enum obj_reloc
   obj_reloc_unhandled
 };
 
-struct obj_string_patch
+struct obj_string_patch_struct
 {
-  struct obj_string_patch *next;
+  struct obj_string_patch_struct *next;
   int reloc_secidx;
   ElfW(Addr) reloc_offset;
   ElfW(Addr) string_offset;
 };
 
-struct obj_symbol_patch
+struct obj_symbol_patch_struct
 {
-  struct obj_symbol_patch *next;
+  struct obj_symbol_patch_struct *next;
   int reloc_secidx;
   ElfW(Addr) reloc_offset;
   struct obj_symbol *sym;
@@ -124,6 +134,35 @@ struct obj_symbol_patch
 
 /* Generic object manipulation routines.  */
 
+#define obj_elf_hash			ObjW(elf_hash)
+#define obj_elf_hash_n			ObjW(elf_hash_n)
+#define obj_add_symbol			ObjW(add_symbol)
+#define obj_find_symbol			ObjW(find_symbol)
+#define obj_symbol_final_value		ObjW(symbol_final_value)
+#define obj_set_symbol_compare		ObjW(set_symbol_compare)
+#define obj_find_section		ObjW(find_section)
+#define obj_insert_section_load_order	ObjW(insert_section_load_order)
+#define obj_create_alloced_section	ObjW(create_alloced_section)
+#define obj_create_alloced_section_first \
+					ObjW(create_alloced_section_first)
+#define obj_extend_section		ObjW(extend_section)
+#define obj_string_patch		ObjW(string_patch)
+#define obj_symbol_patch		ObjW(symbol_patch)
+#define obj_check_undefineds 		ObjW(check_undefineds)
+#define obj_allocate_commons		ObjW(allocate_commons)
+#define obj_load_size			ObjW(load_size)
+#define obj_relocate			ObjW(relocate)
+#define obj_load			ObjW(load)
+#define obj_free			ObjW(free)
+#define obj_create_image		ObjW(create_image)
+#define arch_new_file			ObjW(arch_new_file)
+#define arch_new_section		ObjW(arch_new_section)
+#define arch_new_symbol			ObjW(arch_new_symbol)
+#define arch_apply_relocation		ObjW(arch_apply_relocation)
+#define arch_create_got			ObjW(arch_create_got)
+#define arch_init_module		ObjW(arch_init_module)
+#define arch_load_proc_section		ObjW(arch_load_proc_section)
+
 unsigned long obj_elf_hash(const char *);
 
 unsigned long obj_elf_hash_n(const char *, unsigned long len);
