@@ -39,9 +39,12 @@
      More flexible recognition of the way the utility was called.
      Suggested by Stepan Kasal, implemented in a different way by Keith
      Owens <kaos@ocs.com.au> December 1999.
+ 
+     Rationalize common code for 32/64 bit architectures.
+       Keith Owens <kaos@ocs.com.au> December 1999.
    */
 
-#ident "$Id: insmod.c 1.9 Sat, 11 Dec 1999 04:17:50 +1100 keith $"
+#ident "$Id: insmod.c 1.10 Sun, 19 Dec 1999 20:23:21 +1100 keith $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -65,16 +68,6 @@
 #include "modstat.h"
 #include "config.h"
 
-#if defined(COMMON_sparc)
-#if defined(ARCH_sparc)
-#define main insmod32_main
-#elif defined(ARCH_sparc64)
-#define main insmod64_main
-#endif
-#elif defined(COMBINE_rmmod) || defined(COMBINE_modprobe) || defined(COMBINE_ksyms) || defined(COMBINE_lsmod)
-#define main insmod_main
-#endif
-
 #define STRVERSIONLEN	32
 
 /*======================================================================*/
@@ -89,6 +82,14 @@ static int flag_ksymoops = 1;
 static int n_ext_modules_used;
 static int m_has_modinfo;
 
+extern int insmod_main(int argc, char **argv);
+extern int insmod_main_32(int argc, char **argv);
+extern int insmod_main_64(int argc, char **argv);
+extern int modprobe_main(int argc, char **argv);
+extern int rmmod_main(int argc, char **argv);
+extern int ksyms_main(int argc, char **argv);
+extern int lsmod_main(int argc, char **argv);
+
 /*======================================================================*/
 
 /* Get the kernel version in the canonical integer form.  */
@@ -1061,10 +1062,11 @@ static int old_init_module(const char *m_name, struct obj_file *f, unsigned long
 /* end compat */
 /************************************************************************/
 
-#if defined(COMMON_sparc) && !defined(ARCH_sparc)
-extern void insmod_usage(void);
-#else
-/* Print the usage message. */
+
+/* For common 3264 code, only compile the usage message once, in the 64 bit version */
+#if defined(COMMON_3264) && defined(ONLY_32)
+extern void insmod_usage(void);		/* Use the copy in the 64 bit version */
+#else	/* Common 64 bit version or any non common code - compile usage routine */
 void insmod_usage(void)
 {
 	fputs("Usage:\n"
@@ -1092,9 +1094,17 @@ void insmod_usage(void)
 	      ,stderr);
 	exit(1);
 }
+#endif	/* defined(COMMON_3264) && defined(ONLY_32) */
+
+#if defined(COMMON_3264) && defined(ONLY_32)
+#define INSMOD_MAIN insmod_main_32	/* 32 bit version */
+#elif defined(COMMON_3264) && defined(ONLY_64)
+#define INSMOD_MAIN insmod_main_64	/* 64 bit version */
+#else
+#define INSMOD_MAIN insmod_main		/* Not common code */
 #endif
 
-int main(int argc, char **argv)
+int INSMOD_MAIN(int argc, char **argv)
 {
 	int k_version;
 	int k_crcs;
@@ -1405,88 +1415,91 @@ int main(int argc, char **argv)
 	return exit_status;
 }
 
-#if defined(COMMON_sparc) && defined(ARCH_sparc)
-extern int insmod64_main(int argc, char **argv);
-
-#undef main
-
-#if defined(COMBINE_rmmod) || defined(COMBINE_modprobe) || defined(COMBINE_ksyms) || defined(COMBINE_lsmod)
-#define main insmod_main
-#endif
-
-int main(int argc, char **argv)
+/* For common 3264 code, add an overall insmod_main, in the 64 bit version. */
+#if defined(COMMON_3264) && defined(ONLY_64)
+int insmod_main(int argc, char **argv)
 {
 	struct utsname u;
-	  
-	if (!uname (&u) && !strcmp (u.machine, "sparc64"))
-		return insmod64_main(argc, argv);
+	if (!uname (&u) && !strcmp (u.machine, "64"))
+		return insmod_main_64(argc, argv);
 	else
-		return insmod32_main(argc, argv);
+		return insmod_main_32(argc, argv);
 }
+#endif	/* defined(COMMON_3264) && defined(ONLY_64) */
 
-#endif
-
-#if !defined(COMMON_sparc) || !defined(ARCH_sparc64)
-#if defined(COMBINE_rmmod) || defined(COMBINE_modprobe) || defined(COMBINE_ksyms) || defined(COMBINE_lsmod)
-
-#undef main
-
-#ifdef COMBINE_modprobe
-extern int modprobe_main(int argc, char **argv);
-#endif
-#ifdef COMBINE_rmmod
-extern int rmmod_main(int argc, char **argv);
-#endif
-#ifdef COMBINE_ksyms
-extern int ksyms_main(int argc, char **argv);
-#endif
-#ifdef COMBINE_lsmod
-extern int lsmod_main(int argc, char **argv);
-#endif
 
+/* For common 3264 code, only compile main in the 64 bit version. */
+#if defined(COMMON_3264) && defined(ONLY_32)
+/* Use the main in the 64 bit version */
+#else
+/* This mainline looks at the name it was invoked under, checks that the name
+ * contains exactly one of the possible combined targets and invokes the
+ * corresponding handler for that function.
+ */
 int main(int argc, char **argv)
 {
+	/* List of possible program names and the corresponding mainline routines */
+	static struct { char *name; int (*handler)(int, char **); } mains[] =
+		{
+			{ "insmod", &insmod_main },
+	#ifdef COMBINE_modprobe
+			{ "modprobe", &modprobe_main },
+	#endif
+	#ifdef COMBINE_rmmod
+			{ "rmmod", &rmmod_main },
+	#endif
+	#ifdef COMBINE_ksyms
+			{ "ksyms", &ksyms_main },
+	#endif
+	#ifdef COMBINE_lsmod
+			{ "lsmod", &lsmod_main },
+	#endif
+		};
+	#define MAINS_NO (sizeof(mains)/sizeof(mains[0]))
+	static int mains_match;
+	static int mains_which;
+
 	char *p = strrchr(argv[0], '/');
+	char error_id1[2048] = "The ";		/* Way oversized */
+	char error_id2[2048] = "";		/* Way oversized */
+	int i;
 
 	p = p ? p + 1 : argv[0];
 
-	if (strstr(p, "insmod"))
-		return insmod_main(argc, argv);
-#ifdef COMBINE_modprobe
-	else if (strstr(p, "modprobe"))
-		return modprobe_main(argc, argv);
-#endif
-#ifdef COMBINE_rmmod
-	else if (strstr(p, "rmmod"))
-		return rmmod_main(argc, argv);
-#endif
-#ifdef COMBINE_ksyms
-	else if (strstr(p, "ksyms"))
-		return ksyms_main(argc, argv);
-#endif
-#ifdef COMBINE_lsmod
-	else if (strstr(p, "lsmod"))
-		return lsmod_main(argc, argv);
-#endif
-	else {
-		error("The insmod"
-#ifdef COMBINE_modprobe
-		"/modprobe"
-#endif
-#ifdef COMBINE_rmmod
-		"/rmmod"
-#endif
-#ifdef COMBINE_ksyms
-		"/ksyms"
-#endif
-#ifdef COMBINE_lsmod
-		"/lsmod"
-#endif
-		" combined binary is having an identity crisis.\n"
-		      "Please give it a proper name.");
-		return 1;
+	for (i = 0; i < MAINS_NO; ++i) {
+		if (i) {
+			strcat(error_id1, "/");
+			if (i == MAINS_NO-1)
+				strcat(error_id2, " or ");
+			else
+				strcat(error_id2, ", ");
+		}
+		strcat(error_id1, mains[i].name);
+		strcat(error_id2, mains[i].name);
+		if (strstr(p, mains[i].name)) {
+			++mains_match;
+			mains_which = i;
+		}
 	}
-}
 
-#endif /* Combined executables */
-#endif
+	/* Finish the error identifiers */
+	if (MAINS_NO != 1)
+		strcat(error_id1, " combined");
+	strcat(error_id1, " binary");
+
+	if (mains_match == 0 && MAINS_NO == 1)
+		++mains_match;		/* Not combined, any name will do */
+	if (mains_match == 0) {
+		error("%s does not have a recognisable name, the name must contain one of %s.",
+			error_id1, error_id2);
+		return(1);
+	}
+	else if (mains_match > 1) {
+		error("%s has an ambiguous name, it must contain %s%s.",
+			error_id1, MAINS_NO == 1 ? "" : "exactly one of ", error_id2);
+		return(1);
+	}
+	else
+		return((mains[mains_which].handler)(argc, argv));
+}
+#endif	/* defined(COMMON_3264) && defined(ONLY_32) */
