@@ -25,9 +25,11 @@
  *
  * optind++ in show_module_info, Salvador Ortiz Garcia <sog@msg.com.mx>
  * unified module path handling: Bjorn Ekwall <bj0rn@blox.se> February 1999
+ * Rationalize common code for 32/64 bit architectures.
+ *   Keith Owens <kaos@ocs.com.au> December 1999.
  */
 
-#ident "$Id: modinfo.c 1.4 Sat, 11 Dec 1999 04:17:50 +1100 keith $"
+#ident "$Id: modinfo.c 1.5 Sun, 19 Dec 1999 20:23:21 +1100 keith $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,12 +47,16 @@
 #include "version.h"
 #include "config.h"
 
-#if defined(COMMON_sparc)
-#if defined(ARCH_sparc)
+extern int show_module_info(char *filename, char *fmtstr, int do_parameters);
+extern int show_module_info_32(char *filename, char *fmtstr, int do_parameters);
 extern int show_module_info_64(char *filename, char *fmtstr, int do_parameters);
-#elif defined(ARCH_sparc64)
-#define show_module_info show_module_info_64
-#endif
+
+#if defined(COMMON_3264) && defined(ONLY_32)
+#define SHOW_MODULE_INFO show_module_info_32		/* 32 bit version */
+#elif defined(COMMON_3264) && defined(ONLY_64)
+#define SHOW_MODULE_INFO show_module_info_64		/* 64 bit version */
+#else
+#define SHOW_MODULE_INFO show_module_info		/* Not common 3264 code */
 #endif
 
 static char *get_modinfo_value(struct obj_file *f, const char *key)
@@ -275,7 +281,7 @@ static void show_module_parameters(struct obj_file *f)
 	}
 }
 
-int show_module_info(char *filename, char *fmtstr, int do_parameters)
+int SHOW_MODULE_INFO(char *filename, char *fmtstr, int do_parameters)
 {
 	FILE *fp;
 	struct obj_file *f;
@@ -310,8 +316,23 @@ int show_module_info(char *filename, char *fmtstr, int do_parameters)
 	return 0;
 }
 
-#if !defined(COMMON_sparc) || defined(ARCH_sparc)
-void usage(void)
+/* For common 3264 code, add an overall show_module_info, in the 64 bit version. */
+#if defined(COMMON_3264) && defined(ONLY_64)
+int show_module_info(char *filename, char *fmtstr, int do_parameters)
+{
+	struct utsname u;
+	if (!uname (&u) && !strcmp (u.machine, "64"))
+		return show_module_info_32(filename, fmtstr, do_parameters);
+	else
+		return show_module_info_64(filename, fmtstr, do_parameters);
+}
+#endif  /* defined(COMMON_3264) && defined(ONLY_64) */
+
+/* For common 3264 code, only compile the usage message once, in the 64 bit version */
+#if defined(COMMON_3264) && defined(ONLY_32)
+extern void modinfo_usage(void);	/* Use the copy in the 64 bit version */
+#else	/* Common 64 bit version or any non common code - compile usage routine */
+void modinfo_usage(void)
 {
 	printf("usage: modinfo <parameters> <module>\n");
 	printf("  -a, --author            display module author\n");
@@ -322,7 +343,13 @@ void usage(void)
 	printf("  -V, --version           show version\n");
 	printf("  -h, --help              this usage screen\n");
 }
+#endif  /* defined(COMMON_3264) && defined(ONLY_32) */
 
+
+/* For common 3264 code, only compile main in the 64 bit version. */
+#if defined(COMMON_3264) && defined(ONLY_32)
+/* Use the main in the 64 bit version */
+#else
 int main(int argc, char *argv[])
 {
 	static struct option long_opts[] =
@@ -337,9 +364,6 @@ int main(int argc, char *argv[])
 	};
 	int do_parameters = 0, opt;
 	char *fmtstr = NULL;
-#if defined(COMMON_sparc)
-	struct utsname u;
-#endif
 
 	while ((opt = getopt_long(argc, argv, "adq:pVh",
 				  &long_opts[0], NULL)) != EOF)
@@ -361,26 +385,18 @@ int main(int argc, char *argv[])
 			exit(0);
 		case 'h':
 		default:
-			usage();
+			modinfo_usage();
 			exit(opt == 'h' ? 0 : 1);
 		}
 
 	if (optind >= argc) {
-		usage();
+		modinfo_usage();
 		exit(1);
 	}
 	error_file = "modinfo";
-
-#if defined(COMMON_sparc)
-	if (!uname (&u) && !strcmp (u.machine, "sparc64"))
-	    while (optind < argc)
-		show_module_info_64(argv[optind++], fmtstr, do_parameters);
-	else
-#endif
 	while (optind < argc)
 		show_module_info(argv[optind++], fmtstr, do_parameters);
 
 	return 0;
 }
-
-#endif /* !defined(COMMON_sparc) || defined(ARCH_sparc) */
+#endif  /* defined(COMMON_3264) && defined(ONLY_32) */
