@@ -278,7 +278,7 @@ static LINK *linkit(int last, void *item, LINK *linkto)
 	return me;
 }
 
-static DESC *build_desc(const char *name, const char *objname)
+static DESC *build_desc(const char *name, const char *objname, int allow_alias)
 {
 	DESC *desc;
 	GLOB_LIST *g;
@@ -294,7 +294,7 @@ static DESC *build_desc(const char *name, const char *objname)
 	desc->kname = NULL;
 	desc->nod = objname ? nod : NULL;
 
-	for (p = name; p; p = any_alias(p)) {
+	for (p = name; p; p = allow_alias ? any_alias(p) : NULL) {
 		if (strcmp(p, "off") == 0) {
 			desc->objkey = NULL;
 			break;
@@ -333,7 +333,7 @@ static DESC *build_desc(const char *name, const char *objname)
  *	aliased off	-1
  *	error		-2
  */
-static int build_stack(LINK **stack, const char *name, const char *objname, int dir, DESC *rev, int mode)
+static int build_stack(LINK **stack, const char *name, const char *objname, int dir, DESC *rev, int mode, int allow_alias)
 {
 	NODE *nod;
 	DESC *desc;
@@ -343,7 +343,7 @@ static int build_stack(LINK **stack, const char *name, const char *objname, int
 	int i;
 	int r;
 
-	if ((desc = build_desc(name, objname)) == NULL)
+	if ((desc = build_desc(name, objname, allow_alias)) == NULL)
 		return -2;
 	if (desc->kname == NULL)
 		return -1; /* aliased off */
@@ -372,7 +372,7 @@ static int build_stack(LINK **stack, const char *name, const char *objname, int
 					r = build_stack(stack, g->pathv[i],
 							NULL, dir,
 							rev ? rev : desc,
-							MODE_ABOVE);
+							MODE_ABOVE, 1);
 					if (r < 0)
 						return r;
 				}
@@ -411,7 +411,7 @@ static int build_stack(LINK **stack, const char *name, const char *objname, int
 						NULL,
 						dir,
 						rev,
-						MODE_BELOW)) < 0)
+						MODE_BELOW, 1)) < 0)
 					return r;
 			}
 		}
@@ -419,10 +419,11 @@ static int build_stack(LINK **stack, const char *name, const char *objname, int
 
 	/*
 	 * Get the modules that this one depends on
-	 * according to modules.dep
+	 * according to modules.dep.  These are real modules, alias expansion is
+	 * not allowed.
 	 */
 	for (nod = desc->nod->dep; nod; nod = nod->next) {
-		if ((r = build_stack(stack, nod->key, nod->str, dir, rev, MODE_NORMAL)) < 0)
+		if ((r = build_stack(stack, nod->key, nod->str, dir, rev, MODE_NORMAL, 0)) < 0)
 			return r;
 	}
 
@@ -1159,7 +1160,7 @@ static int unload(const char *name)
 	}
 	/* else */
 
-	ret = build_stack(&stack, mod, NULL, 1, NULL, MODE_NORMAL);
+	ret = build_stack(&stack, mod, NULL, 1, NULL, MODE_NORMAL, 1);
 
 	if (debug)
 		dump_stack(stack);
@@ -1209,7 +1210,7 @@ static int probe_stack(const char *name, const char *obj, char **options, DESC *
 
 	new_in_kernel = stack = NULL;
 
-	if ((status = build_stack(&stack, name, obj, 0, NULL, MODE_NORMAL)) < 0)
+	if ((status = build_stack(&stack, name, obj, 0, NULL, MODE_NORMAL, 1)) < 0)
 		return status;
 
 	if (debug)
