@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_common.c 1.1 Wed, 25 Aug 1999 16:26:49 +1000 keith $"
+#ident "$Id: obj_common.c 1.2 Mon, 18 Oct 1999 16:33:17 +1000 keith $"
 
 #include <string.h>
 #include <assert.h>
@@ -167,7 +167,7 @@ obj_add_symbol (struct obj_file *f, const char *name, unsigned long symidx,
   f->symtab[hash] = sym;
   sym->ksymidx = -1;
 
-  if (ELFW(ST_BIND)(info) == STB_LOCAL)
+  if (ELFW(ST_BIND)(info) == STB_LOCAL && symidx != -1)
     f->local_symtab[symidx] = sym;
 
 found:
