@@ -1,6 +1,8 @@
 /*
  * Get kernel symbol table(s) and other relevant module info.
  *
+ * Add module_name_list and l_module_name_list.
+ *   Keith Owens <kaos@ocs.com.au> November 1999.
  * Bjrn Ekwall <bj0rn@blox.se> in February 1999 (C)
  * Initial work contributed by Richard Henderson <rth@tamu.edu>
  *
@@ -35,11 +37,13 @@
 
 struct module_stat *module_stat;
 size_t n_module_stat;
+char *module_name_list;
+size_t l_module_name_list;
 struct module_symbol *ksyms;
 size_t nksyms;
 int k_new_syscalls;
 
-static void *scratch;
+static void *old_kernsym;
 
 /************************************************************************/
 static void drop(void)
@@ -61,14 +65,19 @@ static void drop(void)
 		module_stat = NULL;
 		n_module_stat = 0;
 	}
+	if (module_name_list) {
+		free(module_name_list);
+		module_name_list = NULL;
+		l_module_name_list = 0;
+	}
 	if (ksyms) {
 		free(ksyms);
 		ksyms = NULL;
 		nksyms = 0;
 	}
-	if (scratch) {
-		free(scratch);
-		scratch = NULL;
+	if (old_kernsym) {
+		free(old_kernsym);
+		old_kernsym = NULL;
 	}
 }
 
@@ -100,7 +109,8 @@ static int new_get_kernel_info(int type)
 		}
 		module_names = xrealloc(module_names, bufsize = ret);
 	}
-	scratch = module_names;
+	module_name_list = module_names;
+	l_module_name_list = bufsize;
 	n_module_stat = nmod = ret;
 	module_stat = modules = xmalloc(nmod * sizeof(struct module_stat));
 	memset(modules, 0, nmod * sizeof(struct module_stat));
@@ -242,13 +252,15 @@ static int old_get_kernel_info(int type)
 	int i;
 
 	drop();
+	module_name_list = xmalloc(1);
+	*module_name_list = '\0';
 
 	if ((nkernsym = get_kernel_syms(NULL)) < 0) {
 		error("get_kernel_syms: %m");
 		return 0;
 	}
 	kernsym = k = xmalloc(nkernsym * sizeof(struct old_kernel_sym));
-	scratch = kernsym;
+	old_kernsym = kernsym;
 	if (get_kernel_syms(kernsym) != nkernsym) {
 		error("inconsistency with get_kernel_syms -- is someone else "
 		      "playing with modules?");
@@ -259,8 +271,16 @@ static int old_get_kernel_info(int type)
 	/* Number of modules */
 	for (k = kernsym, nmod = 0, i = 0; i < nkernsym; ++i, ++k) {
 		if (k->name[0] == '#') {
-			if (k->name[1])
+			if (k->name[1]) {
 				++nmod;
+				i = strlen(k->name+1) + 1;
+				module_name_list =
+					xrealloc(module_name_list,
+					l_module_name_list + i);
+				strcpy(module_name_list+l_module_name_list,
+					k->name+1);
+				l_module_name_list += i;	/* NUL separated strings */
+			}
 			else
 				break;
 		}
